Module es

!!$ Logiciel sous licence CeCILL
!!$ Auteur: Rodolphe Turpault
!!$ Module gérant les initialisations, entrées/sorties, etc.

  Use variables
  Use physique

  Implicit None

Contains


!!$ Lecture du fichier d'entrée init.dat

  Subroutine lit_entree

    Open(1,file='init.dat')
    Read(1,*)L
    Print *,'Longueur du domaine de calcul',L
    Read(1,*)nx
    dx = L/(nx-1)
    Print *,'Pas d''espace',dx
    Read(1,*)cfl
    Read (1,*)tfin
    Print *,'Temps final',tfin
    Read(1,*)clg,cld
    Print *,'Conditions aux limites de type',clg,cld
    Read(1,*)rhog,rhod,ug,ud
    Read(1,*)flunum
    Print *,'Flux numerique de type',flunum
    Read(1,*)init
    Print *,'Initialisation de type',init
    Read(1,*)rhoi,ui
    Print *,'Initialisation avec',rhoi,ui
    Read(1,*)asymptotic
    Print *,'Asymptotique?',asymptotic
    Read(1,*)reprise
    Print *,'Reprise?',reprise
    Read(1,*)naff
    Print *,'Affichage tous les ',naff,' pas de temps'
    Read(1,*)kappa,gamma

    Close(1)

  End Subroutine lit_entree

!!$ Initialisation
  Subroutine initialisation

    Real(PR), Dimension(:), Pointer       :: sigma
    Real(PR), Dimension(:),Pointer        :: cut
    Integer  :: i,j
    Real(PR) :: x


    If (reprise==1) Then
       Open(2,file='reprise.dat')
       Read(2,*)time
       Do i=1,Size(U,2)
          Read(2,*)U(:,i)
       End Do
       Close(2)
    Else
       time = 0._PR
       Select Case(init)  
       Case(1)
!!$ Initialisation par une fonction dans tout le domaine
          Do i=1,nx
             x    = (i-0.5_PR)*dx
             U(1,i) = 0.1_PR*Exp(-(x-0.5_PR*L)**2/0.1_PR)+0.1_PR
             U(2,i) = 0._PR
          End Do
       Case(3)
!!$ Problème de Riemann
          Do i=1,nx
             If ((i-1)*dx<=0.5_PR*L) Then
                U(1,i) = rhog
                U(2,i) = rhog*ug
             Else
                U(1,i) = rhod
                U(2,i) = rhod*ud
             End If
          End Do
       Case(2)
!!$ Initialisation de type créneau
          Do i=1,nx
             If (((i-1)*dx>=0.4_PR*L).And.((i-1)*dx<=0.6_PR*L)) Then
                U(1,i) = rhog
                U(2,i) = rhog*ug
             Else
                U(1,i) = rhod
                U(2,i) = rhod*ud
             End If
          End Do
       Case default
!!$ Constante dans tout le domaine
          U(1,:) = rhoi
          U(2,:) = rhoi*ui
       End Select
    End If

    Allocate(s(nx),hs(nx),p(nx),v(nx),c(nx),pp(nx))

!!$ On remplit le coefficient de friction à partir du fichier sigma.dat
    Open (1,file='sigma.dat')
    Read(1,*)ncf
    If (ncf==0) Then
       Do i=1,nx
          x    = (i-1)*dx
          s(i) = kappa*gamma*(rhoi**gamma)-1._PR/rhoi
       End Do
    Else
       Allocate(sigma(ncf))
       Allocate(cut(ncf+1))
       Read(1,*)sigma(1:ncf)
       Read(1,*)cut(1:ncf+1)
       cut(ncf+1) = L
       
       Do i=1,nx
          x=(i-1)*dx
          Do j=1,ncf
             If ((cut(j)<=x).And.(cut(j+1)>=x)) Then
                s(i) = sigma(j)
             End If
          End Do
       End Do
       Deallocate(sigma)
       Deallocate(cut)
    End If
    Close(1)
    
    Call decode

  End Subroutine initialisation

!!$ Sauvegarde pour reprise et écriture dans des fichiers des résultats
  Subroutine ecriture_sortie

    Integer :: i

    Open(2,file='reprise.dat')
    Write(2,*)time
    Do i=1,Size(U,2)
       Write(2,*)U(:,i)
    End Do
    Close(2)

    Open(12,file='rho.dat')
    Open(13,file='rho_u.dat')
    Open(14,file='u.dat')
    Open(15,file='p.dat')
    Open(16,file='c.dat')
    Do i=1,Size(U,2)
       Write(12,'(ES20.10,2x,ES20.10)')(i-1)*dx,U(1,i)
       Write(13,'(ES20.10,2x,ES20.10)')(i-1)*dx,U(2,i)
       Write(14,'(ES20.10,2x,ES20.10)')(i-1)*dx,v(i)
       Write(15,'(ES20.10,2x,ES20.10)')(i-1)*dx,p(i)
       Write(16,'(ES20.10,2x,ES20.10)')(i-1)*dx,c(i)
    End Do
    Close(12)
    Close(13)
    Close(14)
    Close(15)
    Close(16)

    Deallocate(s,hs,p,v,c,pp)

  End Subroutine ecriture_sortie

End Module es
