Module numerique

!!$ Logiciel sous licence CeCILL
!!$ Auteur: Rodolphe Turpault
!!$ Ce module contient les procdures relies au schma numrique

  Use physique, Only : decode
  Use variables

  Implicit None

Contains

  Subroutine un_pas_temps

    Real(PR), Dimension(:,:), Pointer       :: flu    
    Integer :: i

    Allocate(flu(Size(U,1),2:Size(U,2)))

!!$ On calcule les quantits ncessaires au schma AP 
    Call calcul_alpha_S

!!$ On remplit le flux numrique sur chaque interface
!!$ Convention: flux_num(i)=F_(i-1/2)
    Do i=2,Size(U,2)
       flu(:,i)=flux_num(U(:,i-1),U(:,i),F(:,i-1),F(:,i),bp(i),bm(i))
    End Do

!!$ On fait la mise  jour du vecteur des variables conservatives  l'intrieur du domaine
    Do i=2,Size(U,2)-1
       U1(:,i) =  U(:,i)-dtsdx*(alpha(i+1)*flu(:,i+1)-alpha(i)*flu(:,i))
       U1(:,i) =  U1(:,i)+dtsdx*(alpha(i+1)-alpha(i))*F(:,i)
       U1(:,i) =  U1(:,i)+dtsdx*((1.-alpha(i+1))*Sm(:,i+1)+(1.-alpha(i))*Sp(:,i))
    End Do

!!$ On remplit les conditions aux limites
    Call CLim

    Deallocate(flu)

  End Subroutine un_pas_temps


!!$ La fonction flunum rend le flux numrique d'un schma  deux points de type HLL
!!$ Seuls HLL et Rusanov sont cods ici
  Function flux_num(Ug,Ud,Fg,Fd,bp,bm)

    Real(PR), Dimension(:), Intent(in) :: Ug,Ud,Fg,Fd
    Real(PR), Dimension(Size(Ug,1))    :: flux_num
    Real(PR)                           :: bp,bm

    Select Case (flunum)

    Case (HLL)
       If (bp*bm<=0) Then
          flux_num = (bp*Fg-bm*Fd+bp*bm*(Ud-Ug))/(bp-bm)
       Elseif (bm>0) Then
          flux_num = Fg
       Else
          flux_num = Fd
       End If

    Case default
!!$ schma de Rusanov
       flux_num = 0.5_PR*(Fg+Fd-bp*(Ud-Ug))
    End Select

  End Function flux_num


!!$ Procdure qui remplit les grandeurs ncessaires  la prise en compte du terme source
  Subroutine calcul_alpha_S

    Integer :: i

    Do i=2,Size(U,2)
       alpha(i) = (bp(i)-bm(i))/(bp(i)-bm(i)+dx*(s(i)+hs(i)))
       Sm(:,i)  = (R(:,i-1)-U(:,i-1))*(-Min(0._PR,bm(i))+Min(0._PR,bp(i)))
       Sp(:,i)  = (R(:,i)-U(:,i))*(-Max(0._PR,bm(i))+Max(0._PR,bp(i)))
    End Do

  End Subroutine calcul_alpha_S

!!$ Conditions aux limites
  Subroutine CLim

    Integer :: n

    n = Size(U,2)

!!$ CL  gauche
    Select Case (clg)

    Case(Dirichlet)
       U1(1,1) = rhog
       U1(2,1) = rhog*ug
    Case(sortie)
       U1(:,1) = U1(:,2)
    Case(mur)
       U1(1,1) = U1(1,2)
       U1(2,1) = 0._PR
    Case(miroir)
       U1(1,1) = U1(1,2)
       U1(2,1) = -U1(2,2)
    Case(periodique)
       U1(1,1) = U1(1,n-1)
       U1(2,1) = U1(2,n-1)
    Case default
       Print *,'mauvais type de CL  gauche'
    End Select

!!$ CL  droite
    Select Case (cld)

    Case(Dirichlet)
       U1(1,n) = rhod
       U1(2,n) = rhod*ud
    Case(sortie)
       U1(:,n) = U1(:,n-1)
    Case(mur)
       U1(1,n) = U1(1,n-1)
       U1(2,n) = 0._PR
    Case(miroir)
       U1(1,n) = U1(1,n-1)
       U1(2,n) = -U1(2,n-1)
    Case(periodique)
       U1(1,n) = U1(1,2)
       U1(2,n) = U1(2,2)
    Case default
       Print *,'mauvais type de CL  gauche'
    End Select

  End Subroutine CLim

!!$ Calcul du pas de temps
  Subroutine calcul_dt

    dt    = 0.5_PR*cfl*dx/(Max(Maxval(Abs(bp)),Maxval(Abs(bm))))
    dtsdx = dt/dx

  End Subroutine calcul_dt

End Module numerique
