Module physique

!!$ Logiciel sous licence CeCILL
!!$ Auteur: Rodolphe Turpault
!!$ Module contenant les données relatives au système que l'on approche
!!$ Ici: Euler + friction

  Use variables

  Implicit None

Contains

  Subroutine decode

    Real(PR) :: a,x
    Integer  :: i

!!$ On déduit des variables conservatives la vitesse, pression (et sa dérivée) et vitesse du son
!!$ puis on remplit le flux physique

    Do i=1,Size(U,2)
       v(i)  = U(2,i)/U(1,i)
       p(i)  = kappa*(U(1,i)**gamma)
       c(i)  = Sqrt(gamma*kappa*(U(1,i)**(gamma-1._PR)))
       pp(i) = gamma*kappa*(U(1,i)**(gamma-1._PR))
       
       !!$ Flux physique
       F(1,i)   = U(2,i)
       F(2,i)   = U(2,i)*v(i)+p(i)

    End Do

!!$ On donne éventuellement une expression du coefficient de friction 
!!$ (uniquement si ncf=0 sinon, la valeur par morceaux est lue dans le fichier sigma.dat
    If (ncf==0) Then
       Do i=1,nx
          x    = (i-1)*dx
          s(i) = kappa*gamma*(U(1,i)**gamma)-1._PR/U(1,i)
       End Do
    End If
    
!!$ calcul des vitesses pour HLL/Rusanov
    Do i=2,Size(U,2)
       Select case (flunum)
       Case (HLL)
          a     = Max(U(1,i-1)*c(i-1),U(1,i)*c(i))
          bp(i) = v(i)+a/U(1,i)
          bm(i) = v(i-1)-a/U(1,i-1)
       Case default
!!$ Rusanov
          a     = Max(U(1,i-1)*c(i-1),U(1,i)*c(i))
          bp(i) = max(abs(v(i)+a/U(1,i)),abs(v(i-1)-a/U(1,i-1)))
          bm(i) = -bp(i)
       End Select
    End Do

    Call op_secondaires

!!$ On remplit le second membre
    R(1,1) = U(1,1)
    R(2,1) = 0._PR
    Do i=2,Size(U,2)
       R(1,i)   = U(1,i)
       If ((s(i)+hs(i))/=0._PR) Then
          R(2,i)   = U(2,i)*hs(i)/(hs(i)+s(i))
       Else
          R(2,i) = 0._PR
       End If
    End Do

  End Subroutine decode

!!$ correction du coefficient de friction pour obtenir un schéma AP
!!$ Attention: cette valeur dépend du schéma numérique pour la partie hyperbolique
!!$ Pour HLL et Rusanov, l'expression est la meme.
  Subroutine op_secondaires

    Integer :: i
    
    If (asymptotic) then
       Do i=2,Size(U,2)
          hs(i) = s(i)*(-2._PR*bp(i)*bm(i)/(pp(i-1)+pp(i))-1._PR)
       End Do
    Else
       hs = 0._PR
    End If

  End Subroutine op_secondaires

End Module physique
